#include <stdio.h>
#include <qpushbutton.h>
#include <qtextview.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qcombobox.h>

#if defined (WITH_KDE)
#include <kapp.h>
#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <klocale.h>
#include <ksystemtray.h>
#include "images.h"
#else
#include <qapplication.h>
#endif

#include "imonc.h"
#include "imoncgui.h"
#include "client.h"

Imonc::Imonc(const QString &pass="",
	     const QString &host="fli4l", Q_UINT16 port=5000)
{
    client_ = new Client(host, port);
    client_->start();
    gui_ = new ImoncGui();

#if defined (WITH_KDE)
    tray_ = new KSystemTray(gui_, "icon");
    tray_->setPixmap(QPixmap((const char**) stat_noconnection_xpm));
    tray_->setBackgroundColor(blue);
    tray_->show();
#endif

    QObject::connect(gui_->dialButton, SIGNAL(clicked()),
		     client_, SLOT(dial()));
    QObject::connect(gui_->hangupButton, SIGNAL(clicked()),
		     client_, SLOT(hangup()));
    QObject::connect(gui_->quitButton, SIGNAL(clicked()),
		     qApp, SLOT(quit()));
    QObject::connect(gui_->sendCmdLineEdit, SIGNAL(returnPressed()),
		     this, SLOT(sendCmd()));
    QObject::connect(gui_->openConButton, SIGNAL(clicked()),
		     client_, SLOT(openConnection()));
    QObject::connect(gui_->closeConButton, SIGNAL(clicked()),
		     client_, SLOT(closeConnection()));
    //QObject::connect(this, SIGNAL(setStatusIcon(const QPixmap)),
    //     (QLabel*) tray_, SLOT(setPixmap(const QPixmap)));

    QObject::connect(client_, SIGNAL(statusChanged(int, QString)),
		     this, SLOT(setStatus(int, QString)));
    QObject::connect(client_, SIGNAL(output(QString)),
		     this, SLOT(output(QString)));

    QObject::connect(client_, SIGNAL(setDate(QString)),
		     this, SLOT(setDate(QString)));
    QObject::connect(client_, SIGNAL(setRoute(int)),
		     this, SLOT(setRoute(int)));
    QObject::connect(gui_->circuitComboBox, SIGNAL(activated(int)),
		     client_, SLOT(setNewRoute(int)));
    QObject::connect(client_, SIGNAL(setCircuitList(QStringList)),
		     this, SLOT(setCircuitList(QStringList)));

    QObject::connect(client_, SIGNAL(setCh1Time(QString)),
		     this, SLOT(setCh1Time(QString)));
    QObject::connect(client_, SIGNAL(setCh1ChargeTime(QString)),
		     this, SLOT(setCh1ChargeTime(QString)));
    QObject::connect(client_, SIGNAL(setCh1Charge(QString)),
		     this, SLOT(setCh1Charge(QString)));
    QObject::connect(client_, SIGNAL(setCh1Phone(QString)),
		     this, SLOT(setCh1Phone(QString)));

    QObject::connect(client_, SIGNAL(setCh2Time(QString)),
		     this, SLOT(setCh2Time(QString)));
    QObject::connect(client_, SIGNAL(setCh2ChargeTime(QString)),
		     this, SLOT(setCh2ChargeTime(QString)));
    QObject::connect(client_, SIGNAL(setCh2Charge(QString)),
		     this, SLOT(setCh2Charge(QString)));
    QObject::connect(client_, SIGNAL(setCh2Phone(QString)),
		     this, SLOT(setCh2Phone(QString)));

    status_ = -1;
}


Imonc::~Imonc()
{}

void Imonc::setStatus(int status, QString msg)
{
    QString out;
    out = out.setNum(status);
    gui_->statusLabel->setText(out+" "+msg);
    status_ = status;
    switch (status)
    {
	case -1:
	    gui_->dialButton->setText("n/a");
	    gui_->hangupButton->setText("n/a");
#if defined (WITH_KDE)
	    tray_->setPixmap(QPixmap((const char**) stat_noconnection_xpm));
#if defined (QT_3)
	    tray_->setEraseColor(blue);
#else
	    tray_->setBackgroundColor(blue);
#endif
#endif
	    break;
	case 0:
	    gui_->dialButton->setText("Dial");
	    gui_->hangupButton->setText("Hangup");
	    //gui_->ch1TimeLabel->setPaletteForegroundColor(red);
#if defined (WITH_KDE)
	    tray_->setPixmap(QPixmap((const char**) stat_disconnected_xpm));
#if defined (QT_3)
	    tray_->setEraseColor(blue);
#else
	    tray_->setBackgroundColor(green);
#endif
#endif
	    break;
	case 1:
	    gui_->dialButton->setText("Dial");
	    gui_->hangupButton->setText("Hangup");
#if defined (WITH_KDE)
	    tray_->setPixmap(QPixmap((const char**) stat_connecting_xpm));
#if defined (QT_3)
	    tray_->setEraseColor(blue);
#else
	    tray_->setBackgroundColor(red);
#endif
#endif
	    break;
	case 2:
	    gui_->dialButton->setText("Dial");
	    gui_->hangupButton->setText("Hangup");
#if defined (WITH_KDE)
	    tray_->setPixmap(QPixmap((const char**) stat_disconnecting_xpm));
#if defined (QT_3)
	    tray_->setEraseColor(blue);
#else
	    tray_->setBackgroundColor(green);
#endif
#endif
	    break;
	case 3:
	    gui_->dialButton->setText("Dial");
	    gui_->hangupButton->setText("Hangup");
#if defined (WITH_KDE)
	    tray_->setPixmap(QPixmap((const char**) stat_connected_xpm));
#if defined (QT_3)
	    tray_->setEraseColor(blue);
#else
	    tray_->setBackgroundColor(red);
#endif
#endif
	    break;
	case 4:
	    gui_->dialButton->setText("Dial");
	    gui_->hangupButton->setText("Hangup");
#if defined (WITH_KDE)
	    tray_->setPixmap(QPixmap((const char**) stat_chbundling_xpm));
#if defined (QT_3)
	    tray_->setEraseColor(blue);
#else
	    tray_->setBackgroundColor(red);
#endif
#endif
	    break;
    }
}


void Imonc::output(QString msg)
{
    gui_->serverOutTextView->setText(msg);
    //gui_->serverOutTextView->append(msg);
}


void Imonc::sendCmd()
{
    client_->sendWait(gui_->sendCmdLineEdit->text(), false);
    gui_->sendCmdLineEdit->selectAll();
}


void Imonc::setDate(QString date)
{
    gui_->dateLabel->setText(date);
}


void Imonc::setRoute(int route)
{
    gui_->circuitComboBox->setCurrentItem(route);
}


void Imonc::setCircuitList(QStringList list)
{
    gui_->circuitComboBox->clear();
    gui_->circuitComboBox->insertStringList(list);
}


void Imonc::setCh1Time(QString time)
{
    gui_->ch1TimeLabel->setText(time);
}


void Imonc::setCh1ChargeTime(QString chargeTime)
{
    gui_->ch1ChargeTimeLabel->setText(chargeTime);
}


void Imonc::setCh1Charge(QString charge)
{
    gui_->ch1ChargeLabel->setText(charge);
}


void Imonc::setCh1Phone(QString charge)
{
    gui_->ch1PhoneLabel->setText(charge);
}


void Imonc::setCh2Time(QString time)
{
    gui_->ch2TimeLabel->setText(time);
}


void Imonc::setCh2ChargeTime(QString chargeTime)
{
    gui_->ch2ChargeTimeLabel->setText(chargeTime);
}


void Imonc::setCh2Charge(QString charge)
{
    gui_->ch2ChargeLabel->setText(charge);
}


void Imonc::setCh2Phone(QString charge)
{
    gui_->ch2PhoneLabel->setText(charge);
}


ImoncGui* Imonc::getGui()
{
    return gui_;
}


#if defined (WITH_KDE)
static KCmdLineOptions options[] =
{
    {"host <hostname>", I18N_NOOP(
	"Host where imond is running"), "fli4l" },
    {"port <port>", I18N_NOOP(
	"port imond listens on"), "5000" },
    {"pass <password>", I18N_NOOP(
	"password to connect to imond"), 0},
    { 0, 0, 0 }
};
#endif


int main(int argc, char **argv)
{
    QString host = "fli4l";
    QString port = "5000";
    QString pass;

#if defined (WITH_KDE)
    KAboutData aboutData( "kimonc", I18N_NOOP("Kimonc"),
			  "0.1", "client for fli4l", KAboutData::License_GPL,
			  "(c) 2001, October", 0, "www.fli4l.de", "");
    aboutData.addAuthor("Roger Hartmann", "rhartman@igd.fhg.de");
    KCmdLineArgs::init(argc, argv, &aboutData);
    KCmdLineArgs::addCmdLineOptions(options);
    KCmdLineArgs* args = KCmdLineArgs::parsedArgs();
    host = args->getOption("host");
    port = args->getOption("port");
    pass = args->getOption("pass");
    KApplication a;
#else
    QApplication a(argc, argv);
#endif
    Imonc imonc(pass, host, port.toUInt());

    a.setMainWidget(imonc.getGui());
    imonc.getGui()->show();
    
    return a.exec();
}

